"""Reusable package to interact with Schwinn/Cresta GPS sport watches

.. uml::

  class Device {
    open()
    read()
    read_setting()
    clear()
    close()
  }
  class Reader {
    -_port : pyserial or file object
    -_dump : file object
    read(amount)
  }
  SchwinnReader --|> Reader
  CrestaReader --|> Reader

  Device *-- Reader : > has
  Downloader o-- Device
  Downloader o-- Writer
  Downloader o-- Progress

Tray app
--------

.. uml::
  actor User
  create boundary "Tray app" as Tray
  User --> Tray : launches
  control udev
  Tray -> udev : register Listener
  entity Watch
  User --> Watch : connect watch
  note right : Watch says charging
  Watch --> udev : discovery
  udev -> Tray : device added
  activate Tray
  Tray ->> Tray : check IDs
  create control Downloader
  Tray -> Downloader : fire up
  deactivate Tray
  Downloader ->> Watch : CONNECT
  activate Watch
  Watch -->> Downloader : version info
  note over Watch, Downloader : Watch shows connection screen
  Downloader ->> Downloader : create proper Reader
  opt Reading tracks & waypoints
    Downloader ->> Watch : READ
    Watch -->> Downloader : tracks data
    note right : Watch animates extraction
    note over Watch, Downloader : Watch shows connection screen
  end
  opt Delete tracks
    Downloader ->> Watch : CLEAR
    Watch -->> Downloader : ack
  end
  opt Read settings
    Downloader ->> Watch : READ_SETTINGS
    Watch -->> Downloader : watch settings
  end
  Downloader ->> Watch : DISCONNECT
  Watch -->>Downloader : some stuff
  deactivate Watch
  note over Watch : Watch says charging

.. productionlist:: Schwinn810 message
   Message: Request | Response
   Request: CONNECT | READ | READ_SETTING | CLEAR | DISCONNECT
   Response: Version | Tracks
   Tracks: Summary [Track [Lap ...] ...] (Points ...) (Waypoint ...)

.. uml::
   control Downloader
   entity Device
   Downloader ->> Device : read()
   activate Device
   Device ->> Reader : read_summary()
   activate Reader
   Reader -->> Device : # of tracks, # of waypoints
   deactivate Reader
   Device ->> Device : read tracks info
   activate Device
   loop All tracks
     Device ->> Reader : read track
     activate Reader
     Reader -->> Device : track name, date, # of points
     deactivate Reader
     database Writer
     Device -> Writer : add track
     loop All laps within a track
       Device ->> Reader : read lap
       activate Reader
       Reader -->> Device : lap properties
       deactivate Reader
       Device -> Writer : add lap
     end
   end
   deactivate Device
   loop Tracks with points
     Device -> Writer : begin points of a track
     Device ->> Reader : read_points_summary()
     loop Points of a track
       Device ->> Reader : read_point()
       Reader -->> Device : point data
       Device -> Writer : point
       boundary Progress
       Device -> Progress
     end
     Device -> Writer : commit track
   end
   deactivate Device

"""
from schwinn810.device import Device
from schwinn810.writer_csv import Writer
from schwinn810.progress_text import TextProgress
