#!/usr/bin/env python
'Create TCX history file from lap and point data'

from __future__ import print_function
import argparse, os
from csv import DictReader
from pytz import timezone, utc
from datetime import datetime, timedelta
from utils import read_settings
import constants as config
# import logging
# _log = logging.getLogger(__name__)

def main():
    settings = read_settings()
    settings = settings["csv2tcx"]

    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('--tz',
                        default=settings[config.TZ] if settings[config.TZ] else 'GMT',
                        help='Local time zone')
    parser.add_argument('--nogmt', action='store_true',
                        default=not settings[config.TOGMT] if not settings[config.TOGMT] is None else False,
                        help='Do not convert time stamps to GMT')
    parser.add_argument(dest='path', nargs=1,
                        help='Path to track data')
    parser.add_argument(dest='tcx', nargs='?',
                        help='Output tcx')

    args = parser.parse_args()
    base = os.path.splitext(args.path[0])[0]
    convert(base, args.tcx, args.tz, args.nogmt)

def convert(base, tcx_name=None, tz='GMT', nogmt=False):
    "Convert laps & points CSV into TCX"

    tz = timezone(tz)
    if tcx_name is None:
        tcx_name = base + ".tcx"
    with open(base+".track") as t, open(base+".laps") as l, open(base+".points") as p, \
         open(tcx_name, 'w') as tcx:

        track = next(DictReader(t))
        try:
            track_start = tz.localize(datetime.strptime(track["Start"], "%Y-%m-%d %H:%M:%S"))
        except:                     # support for legacy format with standalone x1
            track_start = tz.localize(datetime.strptime("{:s}:{:s}".format(track["Start"], track["x1"]), "%Y-%m-%d %H:%M:%S"))

#        e = datetime.strptime(track["End"], "%Y-%m-%d %H:%M").replace(tzinfo=tz)
        if nogmt:
            ts = track_start.isoformat()
        else:
            ts = track_start.astimezone(utc).strftime("%Y-%m-%dT%H:%M:%SZ")
        print("""<?xml version="1.0" encoding="UTF-8" standalone="no" ?>
<TrainingCenterDatabase xmlns="http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2 http://www.garmin.com/xmlschemas/TrainingCenterDatabasev2.xsd">

 <Activities>
  <Activity Sport="Running">
        <Id>{:s}</Id>""".format(ts), file=tcx)
        lap_start = track_start
        time_prev = 0.
        kcal_prev = 0
        dist_prev = 0.
        beats_prev = 0

        laps = DictReader(l)
        points = DictReader(p)
        thePoint = next(points)
        for theLap in laps:
            time = float(theLap["Time"])
            kcal = int(float(theLap["kcal"]))
            lap_dist = float(theLap["Distance"])*1.e3
            beats = int(theLap["Beats"])
            if nogmt: # lap_start.astimezone(utc).strftime("%Y-%m-%dT%H:%M:%SZ")
                ts = lap_start.isoformat()
            else:
                ts = lap_start.astimezone(utc).strftime("%Y-%m-%dT%H:%M:%SZ")

            print("""   <Lap StartTime="{:s}">
    <TotalTimeSeconds>{:f}</TotalTimeSeconds>
    <DistanceMeters>{:f}</DistanceMeters>
    <MaximumSpeed>{:s}</MaximumSpeed>
    <Calories>{:d}</Calories>
    <AverageHeartRateBpm><Value>{:d}</Value></AverageHeartRateBpm>""".format(ts, \
                                                                             time - time_prev, \
                                                                             lap_dist - dist_prev, \
                                                                             theLap["MaxSpeed"], \
                                                                             kcal - kcal_prev, \
                                                                             int((beats - beats_prev)/(time - time_prev))), file=tcx)
            time_prev = time
            kcal_prev = kcal
            dist_prev = lap_dist
            beats_prev = beats
            heart_max = int(theLap["MaxHeart"])
            if heart_max>0:
                print("""    <MaximumHeartRateBpm><Value>{:d}</Value></MaximumHeartRateBpm>""".format(heart_max), file=tcx)
            print("""    <Intensity>Active</Intensity>
    <TriggerMethod>Location</TriggerMethod>
            <Track>""", file=tcx)

            while True:
                time = tz.localize(datetime.strptime(thePoint["Time"], "%Y-%m-%d %H:%M:%S"))
                dist = float(thePoint["Distance"])*1.e3
                if dist > lap_dist: # the only reliable next lap criteria
                    lap_start = time # the only source of absolute time
                    break
                if nogmt:
                    ts = time.isoformat()
                else:
                    ts = time.astimezone(utc).strftime("%Y-%m-%dT%H:%M:%SZ")
                print("""     <Trackpoint>
      <Time>{:s}</Time>
      <Position>
       <LatitudeDegrees>{:s}</LatitudeDegrees>
       <LongitudeDegrees>{:s}</LongitudeDegrees>
      </Position>""".format(ts, \
                            thePoint["Latitude"], \
                            thePoint["Longitude"]), file=tcx)
                if thePoint['Elevation']:
                    print("""      <AltitudeMeters>{:s}</AltitudeMeters>""".format(
                            thePoint["Elevation"]), file=tcx)
                print("""      <DistanceMeters>{:f}</DistanceMeters>""".format(dist), file=tcx)
                heart = int(float(thePoint["Heart"]))
                if heart > 0:
                    print("""      <HeartRateBpm><Value>{:d}</Value></HeartRateBpm>
      <SensorState>Present</SensorState>""".format(heart), file=tcx)
                else:
                    print("      <SensorState>Absent</SensorState>", file=tcx)

                print("     </Trackpoint>", file=tcx)

                try:
                    thePoint = next(points)
                except StopIteration:
                    break

            print("""    </Track>
   </Lap>""", file=tcx)

        print("""
   <Creator xsi:type="Device_t">
    <Name>Schwinn 810 GPS</Name>
    <UnitId>2882</UnitId>
    <ProductID>11153</ProductID>
    <Version>
     <VersionMajor>1</VersionMajor>
     <VersionMinor>20</VersionMinor>
     <BuildMajor>0</BuildMajor>
     <BuildMinor>0</BuildMinor>
    </Version>
   </Creator>
  </Activity>
 </Activities>

</TrainingCenterDatabase>""", file=tcx)

if __name__ == '__main__':
    # FORMAT = FORMAT = '%(asctime)-15s %(message)s'
    # logging.basicConfig(level=logging.DEBUG, format=FORMAT)
    main()
